﻿Imports System.IO
Imports System.Drawing.Imaging

Public Class Form1

    ' Select default directories.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim dir_info As DirectoryInfo
        dir_info = New DirectoryInfo( _
            Path.Combine(Application.StartupPath, "..\..\Input"))
        txtInputDir.Text = dir_info.FullName

        dir_info = New DirectoryInfo( _
            Path.Combine(Application.StartupPath, "..\..\Output"))
        txtOutputDir.Text = dir_info.FullName
    End Sub

    ' Let the user select the input directory.
    Private Sub btnPickInputDirectory_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPickInputDirectory.Click
        fbdDirectory.SelectedPath = txtInputDir.Text
        If (fbdDirectory.ShowDialog() = DialogResult.OK) Then
            txtInputDir.Text = fbdDirectory.SelectedPath
        End If
    End Sub

    ' Let the user select the output directory.
    Private Sub btnPickOutputDirectory_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPickOutputDirectory.Click
        fbdDirectory.SelectedPath = txtOutputDir.Text
        If (fbdDirectory.ShowDialog() = DialogResult.OK) Then
            txtOutputDir.Text = fbdDirectory.SelectedPath
        End If
    End Sub

    ' Make the web page and thumbnails.
    Private Sub btnGo_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnGo.Click
        ' Get inputs.
        Dim input_dir As String = txtInputDir.Text
        If Not input_dir.EndsWith("\") Then input_dir &= "\"
        Dim output_dir As String = txtOutputDir.Text
        If Not output_dir.EndsWith("\") Then output_dir &= "\"
        Dim url_prefix As String = txtUrlPrefix.Text
        If ((url_prefix.Length > 0) AndAlso (Not url_prefix.EndsWith("/"))) Then
            url_prefix &= "/"
        End If
        Dim thumb_width As Integer = Integer.Parse(txtThumbWidth.Text)
        Dim thumb_height As Integer = Integer.Parse(txtThumbHeight.Text)

        ' Do the work.
        MakeWebPage(input_dir, output_dir, url_prefix, _
            txtWebPage.Text, thumb_width, thumb_height)
    End Sub

    ' Make the web page and thumbnails.
    Private Sub MakeWebPage(ByVal input_dir As String, ByVal output_dir As String, ByVal url_prefix As String, ByVal web_page As String, ByVal thumb_width As Integer, ByVal thumb_height As Integer)
        ' Open the HTML file.
        Dim html_filename As String = output_dir & web_page
        Using html_file As New StreamWriter(html_filename)
            ' Make a list of the image files.
            Dim files As List(Of String) = _
                FindFiles(input_dir, "*.bmp;*.gif;*.jpg;*.png;*.tif", False)

            ' Process the files.
            For Each image_filename As String In files
                ' Copy the file to the destination directory.
                Dim image_fileinfo As New FileInfo(image_filename)
                Dim dest_filename As String = output_dir & image_fileinfo.Name
                File.Copy(image_filename, dest_filename, True)

                ' Get the image.
                Using bm As New Bitmap(image_filename)
                    ' Get the original size.
                    Dim src_rect As New Rectangle(0, 0, bm.Width, bm.Height)

                    ' Shrink the image.
                    Dim scale As Double = Math.Min( _
                        CDbl(thumb_width) / bm.Width, _
                        CDbl(thumb_height) / bm.Height)
                    Dim shrunk_width As Integer = CInt(bm.Width * scale)
                    Dim shrunk_height As Integer = CInt(bm.Height * scale)
                        dim dest_rect as new Rectangle(0, 0, shrunk_width, shrunk_height)

                    Using thumbnail As New Bitmap(shrunk_width, shrunk_height)
                        ' Copy the image at reduced scale.
                        Using gr As Graphics = Graphics.FromImage(thumbnail)
                            gr.DrawImage(bm, dest_rect, src_rect, GraphicsUnit.Pixel)
                        End Using

                        ' Save the thumbnail image.
                        Dim thumb_filename As String = _
                            dest_filename.Substring(0, _
                            dest_filename.Length - image_fileinfo.Extension.Length) & _
                            "_thumb.png"
                        thumbnail.Save(thumb_filename, ImageFormat.Png)

                        ' Add the thumbnail image to the HTML page.
                        Dim thumb_fileinfo As New FileInfo(thumb_filename)
                        html_file.WriteLine( _
                            "<a href=""" & url_prefix & image_fileinfo.Name & """>" & _
                            "<img src=""" & url_prefix & thumb_fileinfo.Name & """>" & _
                            "</a>")
                    End Using ' Using thumbnail As New Bitmap(shrunk_width, shrunk_height)
                End Using ' Using bm As New Bitmap(image_filename)
            Next image_filename

            ' Close the HTML file.
            html_file.Close()

            MessageBox.Show("Processed " & files.Count & " images.")
        End Using ' Using html_file As New StreamWriter(html_filename)
    End Sub

    ' Search for files matching the patterns.
    Private Function FindFiles(ByVal dir_name As String, ByVal patterns As String, ByVal search_subdirectories As Boolean) As List(Of String)
        ' Make the result list.
        Dim files As New List(Of String)()

        ' Get the patterns.
        Dim pattern_array() As String = patterns.Split(";"c)

        ' Search.
        Dim search_option As SearchOption = SearchOption.TopDirectoryOnly
        If search_subdirectories Then search_option = SearchOption.AllDirectories
        For Each pattern As String In pattern_array
            For Each filename As String In Directory.GetFiles(dir_name, pattern, search_option)
                If Not files.Contains(filename) Then files.Add(filename)
            Next filename
        Next pattern

        ' Sort.
        files.Sort()

        ' Return the result.
        Return files
    End Function
End Class
